@@ -31,20 +31,21 @@ typedef enum {
   MS_PHOSH_NOTIFICATION_URGENCY_CRITICAL,
 } MsPhoshNotificationUrgency;
 
-/**
- * MsPhoshNotifyScreenWakeupFlags:
- * @MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_NONE: No wakeup
- * @MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_URGENCY: Wakeup screen based on notification urgency
- * @MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_CATEGORY: Wakeup screen based on notification category
- * @MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_ANY: Wakeup screen on any notification
- *
- * What notification properties trigger screen wakeup
- */
+
+/* TODO: Use headers from libcbd once that is released or usable as subproject */
 typedef enum {
-  MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_NONE     = 0, /*< skip >*/
-  MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_ANY      = (1 << 0),
-  MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_URGENCY  = (1 << 1),
-  MS_PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_CATEGORY = (1 << 2),
-} MsPhoshNotifyScreenWakeupFlags;
+  MS_CHANNEL_MODE_NONE  = 0,
+  MS_CHANNEL_MODE_COUNTRY  = 1,
+} MsChannelMode;
+
+typedef enum {
+  MS_CHANNEL_LEVEL_UNKNOWN        = 0,
+  MS_CHANNEL_LEVEL_PRESIDENTIAL   = (1 << 0),
+  MS_CHANNEL_LEVEL_EXTREME        = (1 << 1),
+  MS_CHANNEL_LEVEL_SEVERE         = (1 << 2),
+  MS_CHANNEL_LEVEL_PUBLIC_SAFETY  = (1 << 3),
+  MS_CHANNEL_LEVEL_AMBER          = (1 << 4),
+  MS_CHANNEL_LEVEL_TEST           = (1 << 5),
+} MsChannelLevel;
 
 G_END_DECLS
