@@ -6,13 +6,13 @@
 
 #pragma once
 
-#include <adwaita.h>
+#include "ms-panel.h"
 
 G_BEGIN_DECLS
 
 #define MS_TYPE_SENSOR_PANEL (ms_sensor_panel_get_type ())
 
-G_DECLARE_FINAL_TYPE (MsSensorPanel, ms_sensor_panel, MS, SENSOR_PANEL, AdwBin)
+G_DECLARE_FINAL_TYPE (MsSensorPanel, ms_sensor_panel, MS, SENSOR_PANEL, MsPanel)
 
 MsSensorPanel *ms_sensor_panel_new (void);
 
